// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

part of 'sign_up_cubit.dart';

@immutable
class SignUpState {
  final Email email;
  final Password password;
  final FormStatus status;
  final FormData data;
  final String? errorMessage;

  const SignUpState({
    this.email = const Email.pure(),
    this.password = const Password.pure(),
    this.status = FormStatus.pure,
    required this.data,
    this.errorMessage,
  });

  SignUpState copyWith({
    Email? email,
    Password? password,
    FormStatus? status,
    FormData? data,
    String? errorMessage,
  }) {
    return SignUpState(
      email: email ?? this.email,
      password: password ?? this.password,
      status: status ?? this.status,
      data: data ?? this.data,
      errorMessage: errorMessage ?? this.errorMessage,
    );
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
  
    return other is SignUpState &&
      other.email == email &&
      other.password == password &&
      other.status == status &&
      other.data == data &&
      other.errorMessage == errorMessage;
  }

  @override
  int get hashCode {
    return email.hashCode ^
      password.hashCode ^
      status.hashCode ^
      data.hashCode ^
      errorMessage.hashCode;
  }

  @override
  String toString() {
    // ignore: lines_longer_than_80_chars
    return 'SignUpState(email: $email, password: $password, status: $status, data: $data, errorMessage: $errorMessage)';
  }
}
