// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:firebase_database/firebase_database.dart';
import 'package:wyatt_crud_bloc/src/models/model.dart';
import 'package:wyatt_crud_bloc/src/models/queries/queries_interface.dart';
import 'package:wyatt_crud_bloc/src/repositories/crud_repository_interface.dart';

class CrudRepositoryFirebaseDatabase<T> implements CrudRepositoryInterface<T> {
  final FirebaseDatabase _firebaseDatabase;
  final Model<Object, T> _parser;

  late DatabaseReference _rootReference;

  CrudRepositoryFirebaseDatabase(
    String root,
    Model<Object, T> parser, {
    FirebaseDatabase? firebaseDatabase,
  })  : _firebaseDatabase = firebaseDatabase ?? FirebaseDatabase.instance,
        _parser = parser {
    _rootReference = _firebaseDatabase.ref(root);
  }

  @override
  Future<void> create(Model object, {String? id}) {
    DatabaseReference _reference = _rootReference;
    if (id != null) {
      _reference = _reference.child(id);
    }
    return _reference.set(object.toMap());
  }

  @override
  Future<void> delete(String id) {
    final DatabaseReference _reference = _rootReference.child(id);
    return _reference.remove();
  }

  @override
  Future<void> deleteAll() {
    return _rootReference.remove();
  }

  @override
  Future<T?> get(String id) async {
    final DatabaseEvent _event = await _rootReference.child(id).once();
    return _parser.from(_event.snapshot.value);
  }

  @override
  Future<List<T?>> getAll() async {
    final DatabaseEvent _event = await _rootReference.once();
    final List<T?> _objects = [];
    _event.snapshot.children.map((e) => _objects.add(_parser.from(e.value)));
    return _objects;
  }

  @override
  Future<List<T?>> query(List<QueryInterface> conditions) {
    // TODO(hpcl): implement query
    throw UnimplementedError();
  }

  @override
  Stream<List<T?>> stream({String? id, List<QueryInterface>? conditions}) {
    DatabaseReference _reference = _rootReference;
    if (id != null) {
      _reference = _reference.child(id);
    }

    return _reference.onValue.map((e) {
      final List<T?> _objects = [];
      e.snapshot.children.map((e) => _objects.add(_parser.from(e.value)));
      return _objects;
    });
  }

  @override
  Future<void> update(String id, {Model? object, Map<String, dynamic>? raw}) {
    // TODO(hpcl): implement update
    throw UnimplementedError();
  }

  @override
  Future<void> updateAll(Map<String, Object?> raw) {
    // TODO(hpcl): implement updateAll
    throw UnimplementedError();
  }
}
